@@ -113,9 +113,9 @@ extern "C" {
 #include <signal.h>
 
 #ifndef _WIN32
-# include <unistd.h>
 # include <sys/time.h>
 # include <sys/wait.h>
+# include <unistd.h>
 #else
 # define WIN32_LEAN_AND_MEAN
 # include <windows.h>
@@ -189,10 +189,19 @@ extern "C" {
 
 #if __GNUC__ >= 3
 # define expect(expr,value)         __builtin_expect ((expr),(value))
-# define inline                     static inline
+# define inline_size                static inline /* inline for codesize */
+# if EV_MINIMAL
+#  define noinline                  __attribute__ ((noinline))
+#  define inline_speed              static noinline
+# else
+#  define noinline
+#  define inline_speed              static inline
+# endif
 #else
 # define expect(expr,value)         (expr)
-# define inline                     static
+# define inline_speed               static
+# define inline_size                static
+# define noinline
 #endif
 
 #define expect_false(expr) expect ((expr) != 0, 0)
@@ -218,12 +227,13 @@ static int have_monotonic; /* did clock_gettime (CLOCK_MONOTONIC) work? */
 
 static void (*syserr_cb)(const char *msg);
 
-void ev_set_syserr_cb (void (*cb)(const char *msg))
+void
+ev_set_syserr_cb (void (*cb)(const char *msg))
 {
   syserr_cb = cb;
 }
 
-static void
+static void noinline
 syserr (const char *msg)
 {
   if (!msg)
@@ -240,7 +250,8 @@ syserr (const char *msg)
 
 static void *(*alloc)(void *ptr, long size);
 
-void ev_set_allocator (void *(*cb)(void *ptr, long size))
+void
+ev_set_allocator (void *(*cb)(void *ptr, long size))
 {
   alloc = cb;
 }
@@ -322,7 +333,7 @@ ev_time (void)
 #endif
 }
 
-inline ev_tstamp
+ev_tstamp inline_size
 get_clock (void)
 {
 #if EV_USE_MONOTONIC
@@ -375,20 +386,7 @@ ev_now (EV_P)
 
 /*****************************************************************************/
 
-static void
-anfds_init (ANFD *base, int count)
-{
-  while (count--)
-    {
-      base->head   = 0;
-      base->events = EV_NONE;
-      base->reify  = 0;
-
-      ++base;
-    }
-}
-
-void
+void noinline
 ev_feed_event (EV_P_ void *w, int revents)
 {
   W w_ = (W)w;
@@ -405,7 +403,7 @@ ev_feed_event (EV_P_ void *w, int revents)
   pendings [ABSPRI (w_)][w_->pending - 1].events = revents;
 }
 
-static void
+void inline_size
 queue_events (EV_P_ W *events, int eventcnt, int type)
 {
   int i;
@@ -414,7 +412,22 @@ queue_events (EV_P_ W *events, int eventcnt, int type)
     ev_feed_event (EV_A_ events [i], type);
 }
 
-inline void
+/*****************************************************************************/
+
+void inline_size
+anfds_init (ANFD *base, int count)
+{
+  while (count--)
+    {
+      base->head   = 0;
+      base->events = EV_NONE;
+      base->reify  = 0;
+
+      ++base;
+    }
+}
+
+void inline_speed
 fd_event (EV_P_ int fd, int revents)
 {
   ANFD *anfd = anfds + fd;
@@ -435,9 +448,7 @@ ev_feed_fd_event (EV_P_ int fd, int revents)
   fd_event (EV_A_ fd, revents);
 }
 
-/*****************************************************************************/
-
-inline void
+void inline_size
 fd_reify (EV_P)
 {
   int i;
@@ -471,7 +482,7 @@ fd_reify (EV_P)
   fdchangecnt = 0;
 }
 
-static void
+void inline_size
 fd_change (EV_P_ int fd)
 {
   if (expect_false (anfds [fd].reify))
@@ -484,7 +495,7 @@ fd_change (EV_P_ int fd)
   fdchanges [fdchangecnt - 1] = fd;
 }
 
-static void
+void inline_speed
 fd_kill (EV_P_ int fd)
 {
   ev_io *w;
@@ -496,7 +507,7 @@ fd_kill (EV_P_ int fd)
     }
 }
 
-inline int
+int inline_size
 fd_valid (int fd)
 {
 #ifdef _WIN32
@@ -507,7 +518,7 @@ fd_valid (int fd)
 }
 
 /* called on EBADF to verify fds */
-static void
+static void noinline
 fd_ebadf (EV_P)
 {
   int fd;
@@ -519,7 +530,7 @@ fd_ebadf (EV_P)
 }
 
 /* called on ENOMEM in select/poll to kill some fds and retry */
-static void
+static void noinline
 fd_enomem (EV_P)
 {
   int fd;
@@ -533,7 +544,7 @@ fd_enomem (EV_P)
 }
 
 /* usually called after fork if backend needs to re-arm all fds from scratch */
-static void
+static void noinline
 fd_rearm_all (EV_P)
 {
   int fd;
@@ -549,7 +560,7 @@ fd_rearm_all (EV_P)
 
 /*****************************************************************************/
 
-static void
+void inline_speed
 upheap (WT *heap, int k)
 {
   WT w = heap [k];
@@ -566,7 +577,7 @@ upheap (WT *heap, int k)
 
 }
 
-static void
+void inline_speed
 downheap (WT *heap, int N, int k)
 {
   WT w = heap [k];
@@ -590,7 +601,7 @@ downheap (WT *heap, int N, int k)
   ((W)heap [k])->active = k + 1;
 }
 
-inline void
+void inline_size
 adjustheap (WT *heap, int N, int k)
 {
   upheap (heap, k);
@@ -612,7 +623,7 @@ static int sigpipe [2];
 static sig_atomic_t volatile gotsig;
 static ev_io sigev;
 
-static void
+void inline_size
 signals_init (ANSIG *base, int count)
 {
   while (count--)
@@ -642,7 +653,7 @@ sighandler (int signum)
     }
 }
 
-void
+void noinline
 ev_feed_signal_event (EV_P_ int signum)
 {
   WL w;
@@ -675,7 +686,7 @@ sigcb (EV_P_ ev_io *iow, int revents)
       ev_feed_signal_event (EV_A_ signum + 1);
 }
 
-static void
+void inline_size
 fd_intern (int fd)
 {
 #ifdef _WIN32
@@ -687,7 +698,7 @@ fd_intern (int fd)
 #endif
 }
 
-static void
+static void noinline
 siginit (EV_P)
 {
   fd_intern (sigpipe [0]);
@@ -706,11 +717,7 @@ static ev_child *childs [PID_HASHSIZE];
 
 static ev_signal childev;
 
-#ifndef WCONTINUED
-# define WCONTINUED 0
-#endif
-
-static void
+void inline_speed
 child_reap (EV_P_ ev_signal *sw, int chain, int pid, int status)
 {
   ev_child *w;
@@ -725,20 +732,28 @@ child_reap (EV_P_ ev_signal *sw, int chain, int pid, int status)
       }
 }
 
+#ifndef WCONTINUED
+# define WCONTINUED 0
+#endif
+
 static void
 childcb (EV_P_ ev_signal *sw, int revents)
 {
   int pid, status;
 
-  if (0 < (pid = waitpid (-1, &status, WNOHANG | WUNTRACED | WCONTINUED)))
-    {
-      /* make sure we are called again until all childs have been reaped */
-      /* we need to do it this way so that the callback gets called before we continue */
-      ev_feed_event (EV_A_ (W)sw, EV_SIGNAL);
+  /* some systems define WCONTINUED but then fail to support it (linux 2.4) */
+  if (0 >= (pid = waitpid (-1, &status, WNOHANG | WUNTRACED | WCONTINUED)))
+    if (!WCONTINUED
+        || errno != EINVAL
+        || 0 >= (pid = waitpid (-1, &status, WNOHANG | WUNTRACED)))
+      return;
 
-      child_reap (EV_A_ sw, pid, pid, status);
-      child_reap (EV_A_ sw,   0, pid, status); /* this might trigger a watcher twice, but feed_event catches that */
-    }
+  /* make sure we are called again until all childs have been reaped */
+  /* we need to do it this way so that the callback gets called before we continue */
+  ev_feed_event (EV_A_ (W)sw, EV_SIGNAL);
+
+  child_reap (EV_A_ sw, pid, pid, status);
+  child_reap (EV_A_ sw,   0, pid, status); /* this might trigger a watcher twice, but feed_event catches that */
 }
 
 #endif
@@ -774,7 +789,7 @@ ev_version_minor (void)
 }
 
 /* return true if we are running with elevated privileges and should ignore env variables */
-static int
+int inline_size
 enable_secure (void)
 {
 #ifdef _WIN32
@@ -906,7 +921,7 @@ loop_destroy (EV_P)
   /* have to use the microsoft-never-gets-it-right macro */
   array_free (fdchange, EMPTY0);
   array_free (timer, EMPTY0);
-#if EV_PERIODICS
+#if EV_PERIODIC_ENABLE
   array_free (periodic, EMPTY0);
 #endif
   array_free (idle, EMPTY0);
@@ -1052,7 +1067,7 @@ ev_default_fork (void)
 
 /*****************************************************************************/
 
-static int
+int inline_size
 any_pending (EV_P)
 {
   int pri;
@@ -1064,7 +1079,7 @@ any_pending (EV_P)
   return 0;
 }
 
-inline void
+void inline_speed
 call_pending (EV_P)
 {
   int pri;
@@ -1084,7 +1099,7 @@ call_pending (EV_P)
       }
 }
 
-inline void
+void inline_size
 timers_reify (EV_P)
 {
   while (timercnt && ((WT)timers [0])->at <= mn_now)
@@ -1111,8 +1126,8 @@ timers_reify (EV_P)
     }
 }
 
-#if EV_PERIODICS
-inline void
+#if EV_PERIODIC_ENABLE
+void inline_size
 periodics_reify (EV_P)
 {
   while (periodiccnt && ((WT)periodics [0])->at <= ev_rt_now)
@@ -1141,7 +1156,7 @@ periodics_reify (EV_P)
     }
 }
 
-static void
+static void noinline
 periodics_reschedule (EV_P)
 {
   int i;
@@ -1163,7 +1178,7 @@ periodics_reschedule (EV_P)
 }
 #endif
 
-inline int
+int inline_size
 time_update_monotonic (EV_P)
 {
   mn_now = get_clock ();
@@ -1181,7 +1196,7 @@ time_update_monotonic (EV_P)
     }
 }
 
-inline void
+void inline_size
 time_update (EV_P)
 {
   int i;
@@ -1213,7 +1228,7 @@ time_update (EV_P)
               now_floor = mn_now;
             }
 
-# if EV_PERIODICS
+# if EV_PERIODIC_ENABLE
           periodics_reschedule (EV_A);
 # endif
           /* no timer adjustment, as the monotonic clock doesn't jump */
@@ -1227,7 +1242,7 @@ time_update (EV_P)
 
       if (expect_false (mn_now > ev_rt_now || mn_now < ev_rt_now - MAX_BLOCKTIME - MIN_TIMEJUMP))
         {
-#if EV_PERIODICS
+#if EV_PERIODIC_ENABLE
           periodics_reschedule (EV_A);
 #endif
 
@@ -1263,6 +1278,16 @@ ev_loop (EV_P_ int flags)
 
   while (activecnt)
     {
+      /* we might have forked, so reify kernel state if necessary */
+      #if EV_FORK_ENABLE
+        if (expect_false (postfork))
+          if (forkcnt)
+            {
+              queue_events (EV_A_ (W *)forks, forkcnt, EV_FORK);
+              call_pending (EV_A);
+            }
+      #endif
+
       /* queue check watchers (and execute them) */
       if (expect_false (preparecnt))
         {
@@ -1304,7 +1329,7 @@ ev_loop (EV_P_ int flags)
                 if (block > to) block = to;
               }
 
-#if EV_PERIODICS
+#if EV_PERIODIC_ENABLE
             if (periodiccnt)
               {
                 ev_tstamp to = ((WT)periodics [0])->at - ev_rt_now + backend_fudge;
@@ -1323,7 +1348,7 @@ ev_loop (EV_P_ int flags)
 
       /* queue pending timers and reschedule them */
       timers_reify (EV_A); /* relative timers called last */
-#if EV_PERIODICS
+#if EV_PERIODIC_ENABLE
       periodics_reify (EV_A); /* absolute timers called first */
 #endif
 
@@ -1353,14 +1378,14 @@ ev_unloop (EV_P_ int how)
 
 /*****************************************************************************/
 
-inline void
+void inline_size
 wlist_add (WL *head, WL elem)
 {
   elem->next = *head;
   *head = elem;
 }
 
-inline void
+void inline_size
 wlist_del (WL *head, WL elem)
 {
   while (*head)
@@ -1375,7 +1400,7 @@ wlist_del (WL *head, WL elem)
     }
 }
 
-inline void
+void inline_speed
 ev_clear_pending (EV_P_ W w)
 {
   if (w->pending)
@@ -1385,7 +1410,7 @@ ev_clear_pending (EV_P_ W w)
     }
 }
 
-inline void
+void inline_speed
 ev_start (EV_P_ W w, int active)
 {
   if (w->priority < EV_MINPRI) w->priority = EV_MINPRI;
@@ -1395,7 +1420,7 @@ ev_start (EV_P_ W w, int active)
   ev_ref (EV_A);
 }
 
-inline void
+void inline_size
 ev_stop (EV_P_ W w)
 {
   ev_unref (EV_A);
@@ -1494,7 +1519,7 @@ ev_timer_again (EV_P_ ev_timer *w)
     }
 }
 
-#if EV_PERIODICS
+#if EV_PERIODIC_ENABLE
 void
 ev_periodic_start (EV_P_ ev_periodic *w)
 {
@@ -1545,6 +1570,145 @@ ev_periodic_again (EV_P_ ev_periodic *w)
 }
 #endif
 
+#ifndef SA_RESTART
+# define SA_RESTART 0
+#endif
+
+void
+ev_signal_start (EV_P_ ev_signal *w)
+{
+#if EV_MULTIPLICITY
+  assert (("signal watchers are only supported in the default loop", loop == ev_default_loop_ptr));
+#endif
+  if (expect_false (ev_is_active (w)))
+    return;
+
+  assert (("ev_signal_start called with illegal signal number", w->signum > 0));
+
+  ev_start (EV_A_ (W)w, 1);
+  array_needsize (ANSIG, signals, signalmax, w->signum, signals_init);
+  wlist_add ((WL *)&signals [w->signum - 1].head, (WL)w);
+
+  if (!((WL)w)->next)
+    {
+#if _WIN32
+      signal (w->signum, sighandler);
+#else
+      struct sigaction sa;
+      sa.sa_handler = sighandler;
+      sigfillset (&sa.sa_mask);
+      sa.sa_flags = SA_RESTART; /* if restarting works we save one iteration */
+      sigaction (w->signum, &sa, 0);
+#endif
+    }
+}
+
+void
+ev_signal_stop (EV_P_ ev_signal *w)
+{
+  ev_clear_pending (EV_A_ (W)w);
+  if (expect_false (!ev_is_active (w)))
+    return;
+
+  wlist_del ((WL *)&signals [w->signum - 1].head, (WL)w);
+  ev_stop (EV_A_ (W)w);
+
+  if (!signals [w->signum - 1].head)
+    signal (w->signum, SIG_DFL);
+}
+
+void
+ev_child_start (EV_P_ ev_child *w)
+{
+#if EV_MULTIPLICITY
+  assert (("child watchers are only supported in the default loop", loop == ev_default_loop_ptr));
+#endif
+  if (expect_false (ev_is_active (w)))
+    return;
+
+  ev_start (EV_A_ (W)w, 1);
+  wlist_add ((WL *)&childs [w->pid & (PID_HASHSIZE - 1)], (WL)w);
+}
+
+void
+ev_child_stop (EV_P_ ev_child *w)
+{
+  ev_clear_pending (EV_A_ (W)w);
+  if (expect_false (!ev_is_active (w)))
+    return;
+
+  wlist_del ((WL *)&childs [w->pid & (PID_HASHSIZE - 1)], (WL)w);
+  ev_stop (EV_A_ (W)w);
+}
+
+#if EV_STAT_ENABLE
+
+# ifdef _WIN32
+#  undef lstat
+#  define lstat(a,b) _stati64 (a,b)
+# endif
+
+#define DEF_STAT_INTERVAL 5.0074891
+#define MIN_STAT_INTERVAL 0.1074891
+
+void
+ev_stat_stat (EV_P_ ev_stat *w)
+{
+  if (lstat (w->path, &w->attr) < 0)
+    w->attr.st_nlink = 0;
+  else if (!w->attr.st_nlink)
+    w->attr.st_nlink = 1;
+}
+
+static void
+stat_timer_cb (EV_P_ ev_timer *w_, int revents)
+{
+  ev_stat *w = (ev_stat *)(((char *)w_) - offsetof (ev_stat, timer));
+
+  /* we copy this here each the time so that */
+  /* prev has the old value when the callback gets invoked */
+  w->prev = w->attr;
+  ev_stat_stat (EV_A_ w);
+
+  if (memcmp (&w->prev, &w->attr, sizeof (ev_statdata)))
+    ev_feed_event (EV_A_ w, EV_STAT);
+}
+
+void
+ev_stat_start (EV_P_ ev_stat *w)
+{
+  if (expect_false (ev_is_active (w)))
+    return;
+
+  /* since we use memcmp, we need to clear any padding data etc. */
+  memset (&w->prev, 0, sizeof (ev_statdata));
+  memset (&w->attr, 0, sizeof (ev_statdata));
+
+  ev_stat_stat (EV_A_ w);
+
+  if (w->interval < MIN_STAT_INTERVAL)
+    w->interval = w->interval ? MIN_STAT_INTERVAL : DEF_STAT_INTERVAL;
+
+  ev_timer_init (&w->timer, stat_timer_cb, w->interval, w->interval);
+  ev_set_priority (&w->timer, ev_priority (w));
+  ev_timer_start (EV_A_ &w->timer);
+
+  ev_start (EV_A_ (W)w, 1);
+}
+
+void
+ev_stat_stop (EV_P_ ev_stat *w)
+{
+  ev_clear_pending (EV_A_ (W)w);
+  if (expect_false (!ev_is_active (w)))
+    return;
+
+  ev_timer_stop (EV_A_ &w->timer);
+
+  ev_stop (EV_A_ (W)w);
+}
+#endif
+
 void
 ev_idle_start (EV_P_ ev_idle *w)
 {
@@ -1626,79 +1790,8 @@ ev_check_stop (EV_P_ ev_check *w)
   ev_stop (EV_A_ (W)w);
 }
 
-#ifndef SA_RESTART
-# define SA_RESTART 0
-#endif
-
-void
-ev_signal_start (EV_P_ ev_signal *w)
-{
-#if EV_MULTIPLICITY
-  assert (("signal watchers are only supported in the default loop", loop == ev_default_loop_ptr));
-#endif
-  if (expect_false (ev_is_active (w)))
-    return;
-
-  assert (("ev_signal_start called with illegal signal number", w->signum > 0));
-
-  ev_start (EV_A_ (W)w, 1);
-  array_needsize (ANSIG, signals, signalmax, w->signum, signals_init);
-  wlist_add ((WL *)&signals [w->signum - 1].head, (WL)w);
-
-  if (!((WL)w)->next)
-    {
-#if _WIN32
-      signal (w->signum, sighandler);
-#else
-      struct sigaction sa;
-      sa.sa_handler = sighandler;
-      sigfillset (&sa.sa_mask);
-      sa.sa_flags = SA_RESTART; /* if restarting works we save one iteration */
-      sigaction (w->signum, &sa, 0);
-#endif
-    }
-}
-
-void
-ev_signal_stop (EV_P_ ev_signal *w)
-{
-  ev_clear_pending (EV_A_ (W)w);
-  if (expect_false (!ev_is_active (w)))
-    return;
-
-  wlist_del ((WL *)&signals [w->signum - 1].head, (WL)w);
-  ev_stop (EV_A_ (W)w);
-
-  if (!signals [w->signum - 1].head)
-    signal (w->signum, SIG_DFL);
-}
-
-void
-ev_child_start (EV_P_ ev_child *w)
-{
-#if EV_MULTIPLICITY
-  assert (("child watchers are only supported in the default loop", loop == ev_default_loop_ptr));
-#endif
-  if (expect_false (ev_is_active (w)))
-    return;
-
-  ev_start (EV_A_ (W)w, 1);
-  wlist_add ((WL *)&childs [w->pid & (PID_HASHSIZE - 1)], (WL)w);
-}
-
-void
-ev_child_stop (EV_P_ ev_child *w)
-{
-  ev_clear_pending (EV_A_ (W)w);
-  if (expect_false (!ev_is_active (w)))
-    return;
-
-  wlist_del ((WL *)&childs [w->pid & (PID_HASHSIZE - 1)], (WL)w);
-  ev_stop (EV_A_ (W)w);
-}
-
-#if EV_MULTIPLICITY
-void
+#if EV_EMBED_ENABLE
+void noinline
 ev_embed_sweep (EV_P_ ev_embed *w)
 {
   ev_loop (w->loop, EVLOOP_NONBLOCK);
@@ -1729,6 +1822,7 @@ ev_embed_start (EV_P_ ev_embed *w)
 
   ev_set_priority (&w->io, ev_priority (w));
   ev_io_start (EV_A_ &w->io);
+
   ev_start (EV_A_ (W)w, 1);
 }
 
@@ -1740,6 +1834,36 @@ ev_embed_stop (EV_P_ ev_embed *w)
     return;
 
   ev_io_stop (EV_A_ &w->io);
+
+  ev_stop (EV_A_ (W)w);
+}
+#endif
+
+#if EV_FORK_ENABLE
+void
+ev_fork_start (EV_P_ ev_fork *w)
+{
+  if (expect_false (ev_is_active (w)))
+    return;
+
+  ev_start (EV_A_ (W)w, ++forkcnt);
+  array_needsize (ev_fork *, forks, forkmax, forkcnt, EMPTY2);
+  forks [forkcnt - 1] = w;
+}
+
+void
+ev_fork_stop (EV_P_ ev_fork *w)
+{
+  ev_clear_pending (EV_A_ (W)w);
+  if (expect_false (!ev_is_active (w)))
+    return;
+
+  {
+    int active = ((W)w)->active;
+    forks [active - 1] = forks [--forkcnt];
+    ((W)forks [active - 1])->active = active;
+  }
+
   ev_stop (EV_A_ (W)w);
 }
 #endif
