@@ -62,7 +62,7 @@ VARx(struct ev_timer **, timers)
 VARx(int, timermax)
 VARx(int, timercnt)
 
-#if EV_PERIODICS || EV_GENWRAP
+#if EV_PERIODIC_ENABLE || EV_GENWRAP
 VARx(struct ev_periodic **, periodics)
 VARx(int, periodicmax)
 VARx(int, periodiccnt)
@@ -80,5 +80,11 @@ VARx(struct ev_check **, checks)
 VARx(int, checkmax)
 VARx(int, checkcnt)
 
+#if EV_FORK_ENABLE || EV_GENWRAP
+VARx(struct ev_fork **, forks)
+VARx(int, forkmax)
+VARx(int, forkcnt)
+#endif
+
 #undef VARx
 
